#!/usr/bin/env python3
""" Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

import argparse
import subprocess
import sys
import base64
import json
import os
import requests
from pathlib import Path

from cshmac.requests import HmacAuth

from fsr.portal_client import PortalClient

sys.path.append("/opt/cyops/configs/scripts/")
from csadm.logs.logger_defs import get_logger

FSR_FCLOUD_INIT_CONFIG_LOG_FILE = "/var/log/cyops/fcloud/init-config.log"
INIT_CONFIG_START_FILE = "/tmp/.forticloud-init-config-start"
FLAGS_DIR = "/opt/cyops/configs/.flags"
FILE_FSR_INIT_CONFIG_COMPLETED = FLAGS_DIR + "/.forticloud-init-config-completed"
FILE_FSR_INIT_CONFIG_RESTRICT_RERUN = FLAGS_DIR + "/.restrict-rerun-for-forticloud-init-config"

with open('/opt/cyops/configs/keys/APPLIANCE_PUBLIC_KEY', 'r') as private_key_file:
    APPLIANCE_PUBLIC_KEY = private_key_file.read().strip()
with open('/opt/cyops/configs/keys/APPLIANCE_PRIVATE_KEY', 'r') as public_key_file:
    APPLIANCE_PRIVATE_KEY = public_key_file.read().strip()

dir_path = os.path.dirname(FSR_FCLOUD_INIT_CONFIG_LOG_FILE)
if not os.path.exists(dir_path):
    os.makedirs(dir_path)
logger = get_logger(__name__, FSR_FCLOUD_INIT_CONFIG_LOG_FILE)

portal_client = PortalClient(logger)

def download_and_install_license():
    logger.info("Downloading and installing license")
    response = portal_client.download_license()
    if response and response['status_code'] == 200:
        license = response['output']['license_file']
        logger.info("License downloaded successfully")
        return True, portal_client.install_license(license)
    else:
        logger.error("license download failed with status code "+str(response['status_code'])+". Error occurred is: "+response['error'])
    return False, ""

def execute_cmd(cmd, exit_on_failure=True):
    pipes = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    std_out, std_err = pipes.communicate()
    if (pipes.returncode != 0) and exit_on_failure:
        # Exit if the command fails
        logger.error("Failed to execute command [{0}] with error {1}".format(cmd, std_err.strip()))
        exit(1)
    ret = pipes.returncode
    std_out = std_out.strip().decode("utf-8")
    std_err = std_err.strip().decode("utf-8")
    return ret, std_out, std_err

def change_csadmin_ssh_default_password():
    logger.debug("Resetting csadmin user password")
    CSADMIN_ACCOUNT_AGING_INFO_CMD = "chage -l csadmin"
    CSADMIN_PASSWORD_EXPIRE_CMD = "passwd --expire csadmin"
    account_id = portal_client.get_account_id()
    CSADMIN_ACCOUNT_CHANGE_PWD_CMD = "echo -e \""+str(account_id)+"\\n"+str(account_id)+"\""+" | passwd csadmin"
    ret, output, error = execute_cmd(CSADMIN_ACCOUNT_AGING_INFO_CMD)
    output_lines = output.split("\n")
    for item in output_lines:
        if 'Password expires' in item:
            password_expires = item.split(':')[1].strip()
            if password_expires != 'never':
                execute_cmd(CSADMIN_ACCOUNT_CHANGE_PWD_CMD)
                logger.info("csadmin user password reset successful")
                logger.info("Marking password expired for csadmin")
                execute_cmd(CSADMIN_PASSWORD_EXPIRE_CMD)
                logger.info("Marked password expired for csadmin")
            else:
                logger.info("Password reset not required")
            break
    return True, ""

def _get_sni():
    account_id = portal_client.get_account_id()
    portal_url = portal_client.get_portal_url()
    sni = str(account_id) + "." + portal_url.replace("https://", "")
    return sni

def _get_host():
    portal_url = portal_client.get_portal_url()
    return portal_url.replace("https://", "")

def update_sni_and_cert_in_router():
    logger.info("Updating SNI and certificate for router")
    router_url = "https://localhost/api/3/routers/52c5cee8-5c28-4ed2-a886-ec8bf4dc5993"

    sni = _get_sni()
    host = _get_host()
    with open('/opt/cyops/configs/rabbitmq/ssl/cyopsca/cacert.pem') as cacert_file:
        cert = cacert_file.read()
    logger.info("Updating SNI to "+sni)
    method = 'PUT'
    body = {
        "address": host,
        "sni": sni,
        "certificate": cert
    }
    auth =  HmacAuth(router_url, method, APPLIANCE_PUBLIC_KEY, APPLIANCE_PRIVATE_KEY, json.dumps(body))
    response_obj = requests.put(router_url, auth=auth, json=body, verify=False)
    if response_obj.status_code != 200:
        logger.error("Failed to update SNI and certificate for router")
        logger.error(response_obj.text)
    logger.info("Updated SNI and certificate for router")
    return True, ""

def trigger_heartbeat_and_heartbeat_cron_job():
    logger.info("Triggering heartbeat")
    execute_cmd("/opt/cyops/configs/scripts/cloud/openstack/heartbeat.py", exit_on_failure=False)
    logger.info("Triggered heartbeat")

    logger.info("Creating heatbeat cron job")
    HEARTBEAT_CRON_FILE="/etc/cron.d/send_heartbeat"
    try:
        if os.path.isfile(HEARTBEAT_CRON_FILE):
            logger.info("Hearbeat cron already configured")
            return True, ""
        else:
            with open(HEARTBEAT_CRON_FILE, 'w') as hb_cron_file:
                hb_cron_file.write("*/5 * * * * root /opt/cyops/configs/scripts/cloud/openstack/heartbeat.py\n")
            logger.info("Created heatbeat cron job")
            return True, ""
    except Exception as error:
        return  False, ""

def delete_monitor_init_config_cron_job():
    logger.info("Deleting monitor_init_config cron job")
    MONITOR_INIT_CONFIG_CRON_FILE = "/etc/cron.d/monitor-forticloud-init-config"
    if os.path.isfile(MONITOR_INIT_CONFIG_CRON_FILE):
        os.remove(MONITOR_INIT_CONFIG_CRON_FILE)
        logger.info("Deleted monitor_init_config cron job")
    else:
        logger.info("monitor_init_config cron job already deleted")
    return True, ""

def _make_rest_call(url, method, body):
    auth = HmacAuth(url, method, APPLIANCE_PUBLIC_KEY, APPLIANCE_PRIVATE_KEY, json.dumps(body))
    response_obj = requests.request(method, url, auth=auth, json=body, verify=False)
    return response_obj

def change_csadmin_gui_default_password_and_mark_inactive():
    url = "https://localhost/api/3/resetpassword"
    account_id = portal_client.get_account_id()
    body = {
          "uuid": "ce5a842b-fbbc-4c6e-8d35-a0438fcdf80c",
          "currentUUID": "3451141c-bac6-467c-8d72-85e0fab569ce",
          "loginId": "csadmin",
          "password": "Fsr@"+str(account_id),
          "sendEmail": False
        }
    response_obj = _make_rest_call(url, 'POST', body)
    if response_obj.status_code != 200:
        logger.error("Failed to update csadmin GUI password")
        return False, response_obj.text
    logger.info("GUI password for csadmin user changed")
    body = {
          "update": {
            "uuid": "ce5a842b-fbbc-4c6e-8d35-a0438fcdf80c",
            "status": 2,
            "nfa": [
              {
                "service": 1,
                "method": "none",
                "phone": None
              }
            ],
            "user_type": 2
          }
        }
    url = 'https://localhost/api/auth/users'
    response_obj = _make_rest_call(url, 'PUT', body)
    if response_obj.status_code != 200:
        logger.error("Failed to mark csadmin as inactive")
        return False, response_obj.text
    body = {
        "csActive": False
    }
    url = "https://localhost/api/3/people/3451141c-bac6-467c-8d72-85e0fab569ce"
    response_obj = _make_rest_call(url, 'PUT', body)
    if response_obj.status_code != 200:
        logger.error("Failed to mark csadmin as inactive")
        return False, response_obj.text
    logger.info("csadmin user marked inactive")
    return True, ""

def start_initialization():
    Path(INIT_CONFIG_START_FILE).touch()

def end_initialization():
    Path(FILE_FSR_INIT_CONFIG_COMPLETED).touch()

def restrict_rerun():
    Path(FILE_FSR_INIT_CONFIG_RESTRICT_RERUN).touch()
    return True, ""

def change_hostname():
    sni = _get_sni()
    logger.info("Changing hostname to " + sni)
    HOSTNAME_CMD="hostname"
    ret, output, error = execute_cmd(HOSTNAME_CMD)
    if output.strip() == sni:
        logger.info("Hostname already changed")
        return True, ""
    CHANGE_HOSTNAME_CMD = "csadm hostname --set " + sni
    execute_cmd(CHANGE_HOSTNAME_CMD)
    logger.info("hostname changed successfully")
    return True, ""

def regenerate_certs_for_mq():
    logger.info("Regenerating mq certificate")
    MQ_GENERATE_CERT_CMD = "csadm mq --generate-certs"
    execute_cmd(MQ_GENERATE_CERT_CMD)
    logger.info("mq certificate regenerated successfully")
    return True, ""

def refresh_device_uuid():
    cmd = "csadm license --refresh-device-uuid"
    ret, out, err = execute_cmd(cmd, False)
    if ret != 0:
        logger.error("For cmd [" + cmd + "] got out as [" + out + "] and err as [" + err + "]")
        return False, err
    return True, ""

def handle_subsequent_boot():
    steps = [
        "refresh_device_uuid",
        "delete_monitor_init_config_cron_job",
        "restrict_rerun"
    ]
    for step in steps:
        # Keep retrying step until successful
        while True:
            logger.info("Executing subsequent boot step: " + step)
            status, err_msg = globals()[step]()
            if status:
                break
            else:
                logger.error("Subsequent boot step step " + step + " failed with error " + err_msg)
                logger.error("Retrying")

def init_config():
    start_initialization()
    sni = _get_sni()
    logger.info("SNI for the machine is: " + sni)
    init_steps = [
        'change_csadmin_ssh_default_password',
        'download_and_install_license',
        'change_csadmin_gui_default_password_and_mark_inactive',
        'change_hostname',
        'regenerate_certs_for_mq',
        'update_sni_and_cert_in_router',
        'trigger_heartbeat_and_heartbeat_cron_job',
        'delete_monitor_init_config_cron_job',
        'restrict_rerun'
    ]

    for step in init_steps:
        # Keep retrying step until successful
        while True:
            logger.info("Executing step: " + step)
            status, message = globals()[step]()
            if status:
                break
            else:
                logger.error("step " + step + " failed with error " + message)
                logger.error("Retrying")
    end_initialization()
    sys.exit(0)

def define_parser(parser):
    parser.add_argument("--subsequent-boot",
                        dest="is_subsequent_boot",
                        default=False,
                        action="store_true",
                        help="Execute subsquent boot steps")

if __name__ == "__main__":
    if os.path.isfile(FILE_FSR_INIT_CONFIG_RESTRICT_RERUN):
        logger.error("File [" + FILE_FSR_INIT_CONFIG_RESTRICT_RERUN + "] exists. Exiting with 1.")
        sys.exit(1)
    parser = argparse.ArgumentParser()
    define_parser(parser)
    args = parser.parse_args()
    if args.is_subsequent_boot:
        handle_subsequent_boot()
        sys.exit(0)
    init_config()

